<?php
namespace INLINECONTENT\Inlinecontent\Hooks;

use TYPO3\CMS\Backend\Form\FormDataCompiler;
use TYPO3\CMS\Backend\Form\FormDataGroup\TcaDatabaseRecord;
use TYPO3\CMS\Backend\View\PageLayoutView;
use TYPO3\CMS\Backend\View\PageLayoutViewDrawItemHookInterface;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Extbase\Object\ObjectManager;
use TYPO3\CMS\Fluid\View\StandaloneView;

class PageLayoutViewDrawItem implements PageLayoutViewDrawItemHookInterface
{
    /**
     * @var array
     */
    protected $supportedContentTypes = array (
  'inlinecontent_inlinecontent' => 
  array (
    0 => 'tx_inlinecontent_inlinecontent',
  ),
);

    /**
     * @var string
     */
    protected $rootPath = 'EXT:inlinecontent/Resources/Private/Backend/';

    /**
     * Preprocesses the preview rendering of a content element.
     *
     * @param PageLayoutView $parentObject
     * @param bool $drawItem
     * @param string $headerContent
     * @param string $itemContent
     * @param array $row
     */
    public function preProcess(PageLayoutView &$parentObject, &$drawItem, &$headerContent, &$itemContent, array &$row)
    {
        if (!isset($this->supportedContentTypes[$row['CType']])) {
            return;
        }

        $contentType = explode('_', $row['CType'], 2);
        $templateKey = GeneralUtility::underscoredToUpperCamelCase($contentType[1]);
        $templatePath = GeneralUtility::getFileAbsFileName($this->rootPath . 'Templates/' . $templateKey . '.html');
        if (!file_exists($templatePath)) {
            return;
        }

        $objectManager = GeneralUtility::makeInstance(ObjectManager::class);
        $view = $objectManager->get(StandaloneView::class);
        $view->setTemplatePathAndFilename($templatePath);
        $view->setLayoutRootPaths([$this->rootPath . 'Layouts/']);
        $view->setPartialRootPaths([$this->rootPath . 'Partials/']);
        
        $formDataGroup = GeneralUtility::makeInstance(TcaDatabaseRecord::class);
        $formDataCompiler = GeneralUtility::makeInstance(FormDataCompiler::class, $formDataGroup);
        $formDataCompilerInput = [
            'tableName' => 'tt_content',
            'vanillaUid' => (int)$row['uid'],
            'command' => 'edit',
        ];
        $result = $formDataCompiler->compile($formDataCompilerInput);
        $processedRow = $this->getProcessedData($result['databaseRow'], $result['processedTca']['columns']);

        $view->assignMultiple(
            [
                'row' => $row,
                'processedRow' => $processedRow,
            ]
        );

        $itemContent = $view->render();
        $drawItem = false;
    }

    /**
     * @param array $databaseRow
     * @param array $processedTcaColumns
     * @return array
     */
    protected function getProcessedData(array $databaseRow, array $processedTcaColumns)
    {
        $processedRow = $databaseRow;
        foreach ($processedTcaColumns as $field => $config) {
            if (!isset($config['children'])) {
                continue;
            }
            $processedRow[$field] = [];
            foreach ($config['children'] as $child) {
                $processedRow[$field][] = $this->getProcessedData($child['databaseRow'], $child['processedTca']['columns']);
            }
        }

        return $processedRow;
    }
}
